package cn.iocoder.yudao.module.infra.service.file;

import cn.hutool.core.io.IoUtil;
import cn.hutool.core.io.file.FileNameUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.io.FileUtils;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.infra.controller.admin.file.vo.file.BusinessFileVO;
import cn.iocoder.yudao.module.infra.controller.admin.file.vo.file.FileCreateReqVO;
import cn.iocoder.yudao.module.infra.controller.admin.file.vo.file.FilePageReqVO;
import cn.iocoder.yudao.module.infra.controller.admin.file.vo.file.FilePresignedUrlRespVO;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.dal.mysql.file.FileMapper;
import cn.iocoder.yudao.module.infra.framework.file.core.client.FileClient;
import cn.iocoder.yudao.module.infra.framework.file.core.client.s3.FilePresignedUrlRespDTO;
import cn.iocoder.yudao.module.infra.framework.file.core.utils.FileTypeUtils;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;

import java.io.*;
import java.nio.file.Files;
import java.util.HashMap;
import java.util.UUID;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.infra.enums.ErrorCodeConstants.FILE_NOT_EXISTS;

/**
 * 文件 Service 实现类
 *
 * @author 芋道源码
 */
@Service
public class FileServiceImpl implements FileService {

    @Resource
    private FileConfigService fileConfigService;

    @Resource
    private FileMapper fileMapper;

    @Value("${file.upload-path}")
    private String uploadPath;

    @Override
    public PageResult<FileDO> getFilePage(FilePageReqVO pageReqVO) {
        return fileMapper.selectPage(pageReqVO);
    }

    @Override
    @SneakyThrows
    public String createFile(String name, String path, byte[] content) {
        // 计算默认的 path 名
        String type = FileTypeUtils.getMineType(content, name);
        if (StrUtil.isEmpty(path)) {
            path = FileUtils.generatePath(content, name);
        }
        // 如果 name 为空，则使用 path 填充
        if (StrUtil.isEmpty(name)) {
            name = path;
        }

        // 上传到文件存储器
        FileClient client = fileConfigService.getMasterFileClient();
        Assert.notNull(client, "客户端(master) 不能为空");
        String url = client.upload(content, path, type);

        // 保存到数据库
        FileDO file = new FileDO();
        file.setConfigId(client.getId());
        file.setName(name);
        file.setPath(path);
        file.setUrl(url);
        file.setType(type);
        file.setSize(content.length);
        fileMapper.insert(file);
        return url;
    }

    @Override
    public Long createFile(FileCreateReqVO createReqVO) {
        FileDO file = BeanUtils.toBean(createReqVO, FileDO.class);
        fileMapper.insert(file);
        return file.getId();
    }

    @Override
    public void deleteFile(Long id) throws Exception {
        // 校验存在
        FileDO file = validateFileExists(id);

        // 从文件存储器中删除
        FileClient client = fileConfigService.getFileClient(file.getConfigId());
        Assert.notNull(client, "客户端({}) 不能为空", file.getConfigId());
        client.delete(file.getPath());

        // 删除记录
        fileMapper.deleteById(id);
    }

    private FileDO validateFileExists(Long id) {
        FileDO fileDO = fileMapper.selectById(id);
        if (fileDO == null) {
            throw exception(FILE_NOT_EXISTS);
        }
        return fileDO;
    }

    @Override
    public byte[] getFileContent(Long configId, String path) throws Exception {
        FileClient client = fileConfigService.getFileClient(configId);
        Assert.notNull(client, "客户端({}) 不能为空", configId);
        return client.getContent(path);
    }

    @Override
    public FilePresignedUrlRespVO getFilePresignedUrl(String path) throws Exception {
        FileClient fileClient = fileConfigService.getMasterFileClient();
        FilePresignedUrlRespDTO presignedObjectUrl = fileClient.getPresignedObjectUrl(path);
        return BeanUtils.toBean(presignedObjectUrl, FilePresignedUrlRespVO.class,
                object -> object.setConfigId(fileClient.getId()));
    }

    @Override
    @SneakyThrows
    public BusinessFileVO createBusinessReturnFile(MultipartFile file) {
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        long timestamp = System.currentTimeMillis();

        String[] fileInfo = new String[4];
        String name = file.getOriginalFilename();
        String path = null;
        byte[] content = IoUtil.readBytes(file.getInputStream());

        // 计算默认的 path 名
        String type = FileTypeUtils.getMineType(content, name);

        StrUtil.isEmpty(path);//path = FileUtils.generatePath(content, name);
        path = userId + "_" + timestamp;
        String beginPath = name.replace(".", "_");
        path = beginPath + "_" + path + "." + FileNameUtil.extName(name);
        // 如果 name 为空，则使用 path 填充
        if (StrUtil.isEmpty(name)) {
            name = path;
        }

        // 上传到文件存储器
        FileClient client = fileConfigService.getMasterFileClient();
        Assert.notNull(client, "客户端(master) 不能为空");
        String url = client.upload(content, path, type);

        // 保存到数据库
        BusinessFileVO fileDo = new BusinessFileVO();
        fileDo.setConfigId(client.getId());
        fileDo.setName(name);
        fileDo.setPath(path);
        fileDo.setUrl(url);
        fileDo.setType(type);
        fileDo.setSize(content.length);
        fileDo.setUploadUserId(userId);


        //如果业务类型是2 ，说明是保存用户签名图片，那么将用户ID存入businessInstanceId中
        /*if( businessType == 2) {
            //先查询当前用户，是否存有签名，如果没有新增，如果存在，则更新url，并删除infra_file_content中对应的记录
            BusinessFileDO businessFileDO = businessFileMapper.selectOneByBusinessInstanceId(businessType, userId.toString()) ;
            if (businessFileDO == null) {
                fileDo.setBusinessInstanceId(userId+"") ;
                businessFileMapper.insert(fileDo);
            }else {
                client.delete(businessFileDO.getPath());
                businessFileDO.setConfigId(client.getId());
                businessFileDO.setName(name);
                businessFileDO.setPath(path);
                businessFileDO.setUrl(url);
                businessFileDO.setType(type);
                businessFileDO.setSize(content.length);
                businessFileDO.setUploadUserId(userId);
                businessFileDO.setBusinessType(businessType) ;
                businessFileMapper.updateById(businessFileDO) ;
                fileDo = businessFileDO ;
            }
        }else {
            businessFileMapper.insert(fileDo);
        }*/

        fileInfo[0] = name;
        fileInfo[1] = fileDo.getUrl();
        fileInfo[2] = fileDo.getType();
        return fileDo;
    }

    @Override
    public BusinessFileVO businessUploadFile(MultipartFile file) throws IOException {
        File files = new File("");
        String filePath = files.getCanonicalPath() + uploadPath;
        String fileName = file.getOriginalFilename();
        fileName = UUID.randomUUID() + "_" + fileName;
        File dir = new File(filePath);
        if (!dir.exists()) {
            dir.mkdirs();
        }
        File fileObj = new File(filePath, fileName);
        file.transferTo(fileObj);
//        把上传上来的文件存储到磁盘上：（指定一个路径）
//        file.transferTo(uploadFile);
        BusinessFileVO fileDo = new BusinessFileVO();
        fileDo.setUrl(filePath+fileName);
        return fileDo;
    }

}
